pub struct BettingAccount {
pub base: BaseAccount,
pub balances_locked: AHashMap<(InstrumentId, Currency), Money>,
}Fields
base: BaseAccountbalances_locked: AHashMap<(InstrumentId, Currency), Money>Per-(instrument, currency) locked balances (transient, not persisted).
Implementations
Sourceimpl BettingAccount
impl BettingAccount
Sourcepub fn new(event: AccountState, calculate_account_state: bool) -> Self
pub fn new(event: AccountState, calculate_account_state: bool) -> Self
Creates a new BettingAccount instance.
Sourcepub fn update_balance_locked(
&mut self,
instrument_id: InstrumentId,
locked: Money,
)
pub fn update_balance_locked( &mut self, instrument_id: InstrumentId, locked: Money, )
Updates the locked balance for the given instrument and currency.
Panics
Panics if locked is negative.
Sourcepub fn clear_balance_locked(&mut self, instrument_id: InstrumentId)
pub fn clear_balance_locked(&mut self, instrument_id: InstrumentId)
Clears all locked balances for the given instrument ID.
Sourcepub fn update_balances(&mut self, balances: &[AccountBalance]) -> Result<()>
pub fn update_balances(&mut self, balances: &[AccountBalance]) -> Result<()>
Updates the account balances, rejecting negative totals.
Errors
Returns an error if any balance has a negative total.
pub const fn is_unleveraged(&self) -> bool
Sourcepub fn balance_impact(
&self,
instrument: &InstrumentAny,
quantity: Quantity,
price: Price,
order_side: OrderSide,
) -> Money
pub fn balance_impact( &self, instrument: &InstrumentAny, quantity: Quantity, price: Price, order_side: OrderSide, ) -> Money
Returns the balance impact for a betting order.
For Sell (back) the impact is the negative stake (quantity).
For Buy (lay) the impact is the negative liability (quantity * (price - 1)).
Panics
Panics if order_side is NoOrderSide.
Sourcepub fn recalculate_balance(&mut self, currency: Currency)
pub fn recalculate_balance(&mut self, currency: Currency)
Recalculates the account balance for the specified currency based on per-instrument locks.
Sourceimpl BettingAccount
impl BettingAccount
Sourcepub fn py_new(event: AccountState, calculate_account_state: bool) -> Self
pub fn py_new(event: AccountState, calculate_account_state: bool) -> Self
Creates a new BettingAccount instance.
Methods from Deref<Target = BaseAccount>
Sourcepub fn base_balance(
&self,
currency: Option<Currency>,
) -> Option<&AccountBalance>
pub fn base_balance( &self, currency: Option<Currency>, ) -> Option<&AccountBalance>
Returns a reference to the AccountBalance for the specified currency, or None if absent.
Panics
Panics if currency is None and self.base_currency is None.
Sourcepub fn base_balance_total(&self, currency: Option<Currency>) -> Option<Money>
pub fn base_balance_total(&self, currency: Option<Currency>) -> Option<Money>
Returns the total Money balance for the specified currency, or None if absent.
Panics
Panics if currency is None and self.base_currency is None.
pub fn base_balances_total(&self) -> IndexMap<Currency, Money>
Sourcepub fn base_balance_free(&self, currency: Option<Currency>) -> Option<Money>
pub fn base_balance_free(&self, currency: Option<Currency>) -> Option<Money>
Returns the free Money balance for the specified currency, or None if absent.
Panics
Panics if currency is None and self.base_currency is None.
pub fn base_balances_free(&self) -> IndexMap<Currency, Money>
Sourcepub fn base_balance_locked(&self, currency: Option<Currency>) -> Option<Money>
pub fn base_balance_locked(&self, currency: Option<Currency>) -> Option<Money>
Returns the locked Money balance for the specified currency, or None if absent.
Panics
Panics if currency is None and self.base_currency is None.
pub fn base_balances_locked(&self) -> IndexMap<Currency, Money>
pub fn base_last_event(&self) -> Option<AccountState>
Sourcepub fn update_balances(&mut self, balances: &[AccountBalance])
pub fn update_balances(&mut self, balances: &[AccountBalance])
Updates the account balances with the provided list of AccountBalance instances.
Note: This method does NOT validate negative balances. Derived account types
(CashAccount, MarginAccount) should perform their own validation in apply():
MarginAccount: allows negative balances (normal for margin trading)CashAccount: rejects negative unlessallow_borrowingis true
pub fn update_commissions(&mut self, commission: Money)
Sourcepub fn commission(&self, currency: &Currency) -> Option<Money>
pub fn commission(&self, currency: &Currency) -> Option<Money>
Returns the total commission for the specified currency.
Sourcepub fn commissions(&self) -> AHashMap<Currency, Money>
pub fn commissions(&self) -> AHashMap<Currency, Money>
Returns a map of all commissions by currency.
Sourcepub fn base_apply(&mut self, event: AccountState)
pub fn base_apply(&mut self, event: AccountState)
Applies an AccountState event, updating balances.
Panics
Panics if event.account_id does not match this account’s ID.
Sourcepub fn base_purge_account_events(
&mut self,
ts_now: UnixNanos,
lookback_secs: u64,
)
pub fn base_purge_account_events( &mut self, ts_now: UnixNanos, lookback_secs: u64, )
Purges all account state events which are outside the lookback window.
Guaranteed to retain at least the latest event.
Panics
Panics if the purging implementation is changed and all events are purged.
Sourcepub fn base_calculate_balance_locked(
&mut self,
instrument: &InstrumentAny,
side: OrderSide,
quantity: Quantity,
price: Price,
use_quote_for_inverse: Option<bool>,
) -> Result<Money>
pub fn base_calculate_balance_locked( &mut self, instrument: &InstrumentAny, side: OrderSide, quantity: Quantity, price: Price, use_quote_for_inverse: Option<bool>, ) -> Result<Money>
Calculates the amount of balance to lock for a new order based on the given side, quantity, and price.
Errors
This function never returns an error (TBD).
Sourcepub fn base_calculate_pnls(
&self,
instrument: &InstrumentAny,
fill: &OrderFilled,
_position: Option<Position>,
) -> Result<Vec<Money>>
pub fn base_calculate_pnls( &self, instrument: &InstrumentAny, fill: &OrderFilled, _position: Option<Position>, ) -> Result<Vec<Money>>
Calculates profit and loss amounts for a filled order.
For cash accounts, this calculates the balance impact of a fill:
- BUY: gain base currency quantity, lose quote currency notional.
- SELL: lose base currency quantity, gain quote currency notional.
Note: Unlike betting accounts, cash accounts do NOT cap to position quantity. The full fill quantity is used for PnL calculation.
Errors
This function never returns an error (TBD).
Sourcepub fn base_calculate_commission(
&self,
instrument: &InstrumentAny,
last_qty: Quantity,
last_px: Price,
liquidity_side: LiquiditySide,
use_quote_for_inverse: Option<bool>,
) -> Result<Money>
pub fn base_calculate_commission( &self, instrument: &InstrumentAny, last_qty: Quantity, last_px: Price, liquidity_side: LiquiditySide, use_quote_for_inverse: Option<bool>, ) -> Result<Money>
Calculates commission fees for a filled order.
Panics
Panics if instrument fees cannot be converted to f64, or if base currency is unavailable for inverse instruments.